Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Stk_xml.StyleSourceCSS styling.
This module has its own log module named "stk_xml_style".
type source = | Document of Iri.t Css.S.rule_ listThe style is taken from the XML document, prepended by a list of CSS rules. The stylesheets are retrieved from <html>/<head>/<link> and <html><head>/<style> nodes from the XML document.
| Rules of Iri.t Css.S.rule_ listThe style is specified by a list of CSS rules.
*)The source for CSS styling.
The property used to store the source of style.
rules_of_css statements expand the CSS statements according to namespaces definitions, expand the nested rules and returns only the rules (i.e. it filters out @-rules).
This class is used to create objects which define the source to be used for CSS styling, through property prop_source. A handler can be connected to a change of the source (using the connect method and Stk.Object.Prop_changed event). The class also defines a style.css_rules method to compute the CSS rules from the source, the XML document and a function to load remote resources.
Create a new style object.
CSS statements containing default property values for all xhtml elements. Source: https://www.w3schools.com/cssref/css_default_values.php.